<?php

class Advanced_Ads_Page_Peel {

	/**
	 * Holds plugin base class
	 *
	 * @var Advanced_Ads_Page_Peel_Plugin
	 */
	protected $plugin;

	/**
	 * Initialize the plugin by setting localization and loading public scripts
	 * and styles.
	 */
	public function __construct( $is_admin, $is_ajax ) {
		$this->plugin = Advanced_Ads_Page_Peel_Plugin::get_instance();

		add_action( 'plugins_loaded', array( $this, 'wp_plugins_loaded_ad_actions' ), 20 );

		if ( ! $is_admin ) {
			add_action( 'plugins_loaded', array( $this, 'wp_plugins_loaded' ) );
		}
	}

	/**
	 * Load actions and filters needed only for ad rendering
	 * this will make sure options get loaded for ajax and non-ajax-calls
	 */
	public function wp_plugins_loaded_ad_actions() {
		// stop, if main plugin doesn’t exist
		if ( ! class_exists( 'Advanced_Ads', false ) ) {
			return;
		}

		// add page peel placement
		add_action( 'advanced-ads-placement-types', array( $this, 'add_page_peel_placement' ) );
		// add options to the wrapper
		add_filter( 'advanced-ads-set-wrapper', array( $this, 'page_peel_css' ), 22, 2 );
		// add wrapper options.
		add_filter( 'advanced-ads-output-wrapper-options', array( $this, 'add_wrapper_options' ), 21, 2 );
		// add wrapper options, group
		add_filter( 'advanced-ads-output-wrapper-options-group', array( $this, 'add_wrapper_options_group' ), 10, 2 );
	}

	/**
	 * Load actions and filters
	 */
	public function wp_plugins_loaded() {
		// stop, if main plugin doesn’t exist
		if ( ! class_exists( 'Advanced_Ads', false ) ) {
			return;
		}

		// append js file into footer
		add_action( 'wp_enqueue_scripts', array( $this, 'footer_scripts' ) );
		// frontend output
		add_action( 'wp_head', array( $this, 'header_output' ) );
		// inject ad content into footer
		add_action( 'wp_footer', array( $this, 'footer_injection' ), 10 );
	}

	/**
	 * Add page peel placement to list of placements (on placement page, but also for all AXAX calls)
	 *
	 * @param array $types existing placements.
	 * @return array $types
	 */
	public function add_page_peel_placement( $types ) {

		// fixed header bar
		$types['page_peel'] = array(
			'title'       => __( 'Page Peel Ads', 'advanced-ads-page-peel' ),
			'description' => __( 'An ad that is partially hidden in the corner of the website and only reveals itself when the user hovers over it.', 'advanced-ads-page-peel' ),
			'image'       => AACP_BASE_URL . 'admin/assets/img/page-peel.png',
			'options'     => array(
				'placement-item-alternative' => false,
			),
		);

		return $types;
	}

	/**
	 * Inject ad placement into footer
	 */
	public function footer_injection() {
		if ( function_exists( 'advads_is_amp' ) && advads_is_amp() ) {
			return;
		}
		$placements = get_option( 'advads-ads-placements', array() );
		if ( is_array( $placements ) ) {
			foreach ( $placements as $_placement_id => $_placement ) {
				if ( isset( $_placement['type'] ) && in_array( $_placement['type'], array( 'page_peel' ), true ) ) {
					echo Advanced_Ads_Select::get_instance()->get_ad_by_method( $_placement_id, Advanced_Ads_Select::PLACEMENT );
				}
			}
		}
	}

	/**
	 * Add page peel  attributes to wrapper
	 *
	 * @param array           $options ad options.
	 * @param Advanced_Ads_Ad $ad ad object.
	 */
	public function add_wrapper_options( $options, Advanced_Ads_Ad $ad ) {
		$top_level = ! isset( $ad->args['previous_method'] ) || 'placement' === $ad->args['previous_method'];
		if ( ! $top_level ) {
			return $options; }

		// new settings from the ad itself
		$width  = ( isset( $ad->width ) ) ? $ad->width : 0;
		$height = ( isset( $ad->height ) ) ? $ad->height : 0;
		return $this->get_wrapper_options( $options, $ad->args, $width, $height );
	}


	/**
	 * Add page peel  attributes to group wrapper.
	 *
	 * @param array              $options Existing attributes.
	 * @param Advanced_Ads_Group $group Advanced_Ads_Group.
	 */
	public function add_wrapper_options_group( $options, Advanced_Ads_Group $group ) {
		$top_level = ! isset( $group->ad_args['previous_method'] ) || 'placement' === $group->ad_args['previous_method'];
		if ( ! $top_level ) {
			return $options; }

		$width     = ! empty( $group->ad_args['placement_width'] ) ? absint( $group->ad_args['placement_width'] ) : 0;
		$height    = ! empty( $group->ad_args['placement_height'] ) ? absint( $group->ad_args['placement_height'] ) : 0;
		$add_width = $group->type === 'slider' && $width;

		return $this->get_wrapper_options( $options, $group->ad_args, $width, $height, $add_width );
	}

	/**
	 * Get wrapper attributes.
	 *
	 * @param array $options Existing attributes.
	 * @param array $args Arguments passed to ads.
	 * @param int   $width Width of the wrapper.
	 * @param int   $height Height of the wrapper.
	 * @return array $options Modified attributes.
	 */
	private function get_wrapper_options( $options, $args, $width, $height ) {
		if ( isset( $args['placement_type'] ) && $args['placement_type'] === 'page_peel' ) {
			$page_peel_class       = $this->get_page_peel_class();
			$options['class'][] = $page_peel_class . ' page-peel-' . $args['previous_id'] . ' page-peel-transition';

			$full_width               = isset( $args['page_peel']['full_width'] ) && $args['page_peel']['full_width'] !== '' ? $args['page_peel']['full_width'] : $width;
			$full_height              = isset( $args['page_peel']['full_height'] ) && $args['page_peel']['full_height'] !== '' ? $args['page_peel']['full_height'] : $height;
			$options['data-width'][]  = $full_width;
			$options['data-height'][] = $full_height;

			$options['data-id'] = $args['previous_id'];
			$options['class'][] = $page_peel_class . '-onload';

			if ( isset( $args['page_peel']['layout'] ) && $args['page_peel']['layout'] === 'rectangle' ) {
				$options['class'][] = 'advads-page-peel-show-in-rectangle';
			}

			$position                     = ! empty( $args['page_peel']['position'] ) ? $args['page_peel']['position'] : 'topright';
			$options['style']['display']  = 'none';
			$options['style']['z-index']  = '9999';
			$options['style']['overflow']  = 'hidden';
			$options['style']['position'] = 'fixed';

			$top_pos = is_admin_bar_showing() ? '32px' : 0;

			switch ( $position ) {
				case 'topleft':
					$options['style']['top']  = $top_pos;
					$options['style']['left'] = 0;
					break;
				case 'topright':
					$options['style']['top']   = $top_pos;
					$options['style']['right'] = 0;
					break;
				case 'bottomleft':
					$options['style']['bottom'] = 0;
					$options['style']['left']   = 0;
					break;
				case 'bottomright':
					$options['style']['bottom'] = 0;
					$options['style']['right']  = 0;
					break;
			}

			$options['class'][] = 'page-peel-' . $position;
		}

		return $options;
	}

	/**
	 * Add page peel placement args to CSS
	 *
	 * @param array $wrapper wrapper options.
	 * @param $ad
	 * @return array
	 */
	public function page_peel_css( $wrapper, $ad ) {
		if ( ! is_admin() ) {
			$args         = $ad->args;
			$start_width  = isset( $args['page_peel']['start_width'] ) && $args['page_peel']['start_width'] !== '' ? $args['page_peel']['start_width'] : 58;
			$start_height = isset( $args['page_peel']['start_height'] ) && $args['page_peel']['start_height'] !== '' ? $args['page_peel']['start_height'] : 58;
			$full_width   = ! empty( $args['page_peel']['full_width'] ) ? absint( $args['page_peel']['full_width'] ) : $ad->width;
			$full_height  = ! empty( $args['page_peel']['full_height'] ) ? absint( $args['page_peel']['full_height'] ) : $ad->height;
			$placement_id = isset( $args['previous_id'] ) && $args['previous_id'] !== '' ? $args['previous_id'] : '';
			$peel_color   = isset( $args['page_peel']['peel_color'] ) && $args['page_peel']['peel_color'] !== '' ? $args['page_peel']['peel_color'] : '#5d5d5d';

			$style = '<style type="text/css">
					.page-peel-' . $placement_id . ', .page-peel-' . $placement_id . ' + .peel-shadow {
					  width: ' . $start_width . 'px;
					  height: ' . $start_height . 'px; }
					.page-peel-' . $placement_id . ':hover{
					  width: ' . $full_width . 'px;
				  	  height: ' . $full_height . 'px; }
					.page-peel-' . $placement_id . ':hover + .peel-shadow {
					  width: ' . ( $full_width + $full_width * 0.05 ) . 'px;
					  height: ' . ( $full_height + $full_height * 0.05 ) . 'px; }
				  	.page-peel-' . $placement_id . '.page-peel-topright:before {
				  	  border-style: solid;
					  border-width: ' . $start_height . 'px 0 0 ' . $start_width . 'px;
					  border-color: transparent transparent transparent ' . $peel_color . '; }
				  	.page-peel-' . $placement_id . '.page-peel-topright:hover:before {
					  border-width: ' . $full_height . 'px 0 0 ' . $full_width . 'px; }
					.page-peel-' . $placement_id . '.page-peel-topleft:before {
				  	  border-style: solid;
					  border-width: 0 0 ' . $start_height . 'px ' . $start_width . 'px;
					  border-color: transparent transparent ' . $peel_color . ' transparent; }
				  	.page-peel-' . $placement_id . '.page-peel-topleft:hover:before {
					  border-width: 0 0 ' . $full_height . 'px ' . $full_width . 'px; }
					.page-peel-' . $placement_id . '.page-peel-bottomright:before {
				  	  border-style: solid;
					  border-width: ' . $start_height . 'px ' . $start_width . 'px 0 0;
					  border-color: ' . $peel_color . ' transparent transparent transparent; }
				  	.page-peel-' . $placement_id . '.page-peel-bottomright:hover:before {
					  border-width: ' . $full_height . 'px ' . $full_width . 'px 0 0; }
					.page-peel-' . $placement_id . '.page-peel-bottomleft:before {
				  	  border-style: solid;
					  border-width: 0 ' . $start_width . 'px ' . $start_height . 'px 0;
					  border-color: transparent ' . $peel_color . ' transparent transparent; }
				  	.page-peel-' . $placement_id . '.page-peel-bottomleft:hover:before {
					  border-width: 0 ' . $full_width . 'px ' . $full_height . 'px 0; }
				  </style>';

			echo $style;
		}

		return $wrapper;
	}

	/**
	 * Append js file in footer
	 */
	public function footer_scripts() {
		if ( function_exists( 'advads_is_amp' ) && advads_is_amp() ) {
			return;
		}

		$deps = array( 'jquery' );

		if ( class_exists( 'Advanced_Ads_Pro' ) ) {
			$pro_options = Advanced_Ads_Pro::get_instance()->get_options();
			if ( ! empty( $pro_options['cache-busting']['enabled'] ) ) {
				$deps[] = 'advanced-ads-pro/cache_busting';
			}
		}

		wp_enqueue_style( 'page-peel', AACP_BASE_URL . 'public/assets/css/page-peel.css', array(), '1' );

		wp_enqueue_script( 'advanced-ads-page-peel-footer-js', AACP_BASE_URL . 'public/assets/js/page-peel.js', $deps, AACP_VERSION, true );
		wp_localize_script( 'advanced-ads-page-peel-footer-js', 'Advanced_Ads_Page_Peel_settings', array(
			'page_peel_class' => $this->get_page_peel_class(),
		) );
	}

	/**
	 * Content output in the header
	 */
	public function header_output() {
		// inject js array for banner conditions
		if ( function_exists( 'advads_is_amp' ) && advads_is_amp() ) {
			return;
		}
		echo '<script>advads_page_peel_items = { conditions: {}, showed: [] };</script>';
	}

	/**
	 * Returns the (css) class name for page ads
	 */
	final public static function get_page_peel_class() {
		return Advanced_Ads_Plugin::get_instance()->get_frontend_prefix() . 'page-peel';
	}
}
